<?php

class DispletRetsIdxRewriteController extends DispletRetsIdxPlugin {
	public static function check_flush_rewrite_rules() {
		$flush_rewrite_version = get_option( 'displet_rets_idx_flushed_rewrite_rules' );
		if ( empty( $flush_rewrite_version ) ){
			global $wp_rewrite;
			$wp_rewrite->flush_rules();
			update_option( 'displet_rets_idx_flushed_rewrite_rules', true );
		}
	}

	public static function generate_rewrite_rules() {
		$page_slug = trailingslashit( DispletRetsIdxUtilities::get_full_page_slug( self::$_options['property_details_page_id'] ) );
		add_rewrite_tag( '%property_state%', '([^/]{1,3})' );
		add_rewrite_tag( '%property_city%', '([^/]+)' );
		add_rewrite_tag( '%property_zip%', '([^/]+)' );
		add_rewrite_tag( '%property_id%', '([^/]+)' );
		add_rewrite_tag( '%property_address%', '([^/]+)' );
		add_rewrite_tag( '%property_price%', '([\d]+)' );
		add_rewrite_tag( '%property_update_id%', '([^/]+)' );
		add_rewrite_tag( '%re_user_login%', '([^/]+)' );
		add_rewrite_tag( '%residential_permalinks%', '([^/]+)' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/([^/]*)/page/([0-9]+)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]&property_zip=$matches[3]&paged=$matches[4]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/page/([0-9]+)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]&paged=$matches[3]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/page/([0-9]+)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&paged=$matches[2]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/([^/]*)/([^/]*)/([^/]*)/([\d]*)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]&property_zip=$matches[3]&property_id=$matches[4]&property_address=$matches[5]&property_price=$matches[6]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/([^/]*)/([^/]*)/([^/]*)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]&property_zip=$matches[3]&property_id=$matches[4]&property_address=$matches[5]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/([^/]*)/([^/]*)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]&property_zip=$matches[3]&property_id=$matches[4]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/([^/]*)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]&property_zip=$matches[3]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/([^/]*)/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]&property_city=$matches[2]', 'top' );
		add_rewrite_rule( $page_slug . '([^/]{1,3})/?$', 'index.php?page_id=' . self::$_options['property_details_page_id'] . '&property_state=$matches[1]', 'top' );
		add_rewrite_rule('rets-mobile/([^/]{1,3})/?([^/]*)/?([^/]*)/?([^/]*)/?([^/]*)/?([\d]*)/?$', 'index.php?post_type=rets-mobile&property_state=$matches[1]&property_city=$matches[2]&property_zip=$matches[3]&property_id=$matches[4]&property_address=$matches[5]&property_price=$matches[6]', 'top');
		add_rewrite_rule('displet-property-update/([^/]+)/?([^/]*)/?$', 'index.php?property_update_id=$matches[1]&re_user_login=$matches[2]', 'top');
	}

	public static function maybe_reset_flush_rewrite( $post_id ){
		if ( $post_id == self::$_options['property_details_page_id'] ) {
			self::reset_flush_rewrite_rules();
		}
	}

	public static function reset_flush_rewrite_rules() {
		update_option( 'displet_rets_idx_flushed_rewrite_rules', false );
	}
}

?>